local _G = _G
local LBF = LibStub("LibButtonFacade", true)
local elements = {"Earth", "Air", "Water", "Fire"}
function Yata:CreateBarTotemOrder()
	if not self.db.profile.BarTotemOrder then
		self.db.profile.BarTotemOrder = { ["ElementsOrder"] = {} }	
		for k,v in ipairs(elements) do
			self.db.profile.BarTotemOrder[v] = {}
			self.db.profile.BarTotemOrder.ElementsOrder[k] = v	
		end
	end
	-- removalTable
	local goodTable = {}
	for k,v in ipairs(elements) do
		goodTable[v] = {}
	end
	for k,v in ipairs(Yata.TotemData) do
		local totem = Yata.TotemData[v]
		totem = totem[#totem]
		local found = #goodTable[totem.Element] + 1
		for m,n in ipairs(self.db.profile.BarTotemOrder[totem.Element] or {}) do
			if n == totem.Name then
				found = m
				break
			end
		end
		table.insert(goodTable[totem.Element],found,totem.Name)
	end	
	for k,v in ipairs(elements) do
		self.db.profile.BarTotemOrder[v] = {}
		for m,n in pairs(goodTable[v]) do		
			table.insert(self.db.profile.BarTotemOrder[v], n)
		end
	end
end

function Yata:SetButtonSkin(skin)
	self.db.profile.ButtonSkin = skin or self.db.profile.ButtonSkin
	if LBF and self.LBFGroup then
		self.LBFGroup:Skin(skin,0,true)
	end
end

function Yata:SetAutoButtonSwap(autobutton)	
	self.db.profile.AutoButtonSwap = autobutton or self.db.profile.AutoButtonSwap
	for i,b in ipairs(self.TotemButtons.Buttons) do
		b:SetOnClickScript(self.db.profile.AutoButtonSwap)
	end
end

function Yata:SetBarConfiguration(gapValue, orientationValue, visibleValue)
	gapValue = gapValue or self.db.profile.ButtonGap
	orientationValue = orientationValue or self.db.profile.BarOrientation
	visibleValue = visibleValue or self.db.profile.VisibleButtons
	self.db.profile.ButtonGap = gapValue
	self.db.profile.BarOrientation = orientationValue
	self.db.profile.VisibleButtons = visibleValue
	if self.db.profile.ButtonSkin == "Blizzard" then
		gapValue = gapValue + 4
	end
	local buttonSize =  36 + gapValue
	for k,v in ipairs(self.db.profile.BarTotemOrder.ElementsOrder) do
		local header = self.TotemButtons[v].Header
		header:SetAttribute("buttongap",gapValue)
		header:SetWidth(36)
		header:SetHeight(36)
		header:SetAttribute("visiblebuttons", visibleValue)
		header:SetAttribute("orientation",orientationValue)
		local orientation = orientationValue
		local anchor = nil
		if orientation == 1 then
			anchor = "BOTTOMLEFT"
		elseif orientation == 2 then
			anchor = "TOPLEFT"
		end
		header:ClearAllPoints()
		header:SetPoint(anchor, self.TotemBarGrip,anchor,(k-1)*buttonSize,0)
	end
	self.TotemBarGrip:SetWidth(#self.db.profile.BarTotemOrder.ElementsOrder * buttonSize)
	for k,v in ipairs(self.TotemButtons.Buttons) do
		v:SetAttribute("newposition", v:GetAttribute("position"))
	end
end

function Yata:SkinChanged(SkinID,Gloss,Backdrop,Group,Button,Colors)
	self.db.profile.ButtonSkin = SkinID
	self:SetBarConfiguration()
end

function Yata:CreateTotemBar()
	self:CreateBarTotemOrder()
	if LBF then
		self.LBFGroup = LBF:Group("Yata")
		self.LBFGroup.SkinID = self.db.profile.ButtonSkin
		LBF:RegisterSkinCallback("Yata",self.SkinChanged,self)		
	end	
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN", "BarUpdateHandler")
	self.TotemBarGrip = Yata:CreateBarGrip()
	self.TotemButtons =  {["Buttons"] = {}, ["Earth"] = { ["Header"] = nil, ["Buttons"] = {} }, ["Fire"] = {["Header"] = nil, ["Buttons"] = {} }, ["Water"] = {["Header"] = nil, ["Buttons"] = {} }, ["Air"] = {["Header"] = nil, ["Buttons"] = {} }}
	for k,v in ipairs(self.db.profile.BarTotemOrder.ElementsOrder) do
		local header = Yata.Button:CreateHeader(v, self.TotemBarGrip)
		self.TotemButtons[v].Header = header
		for m,n in ipairs(self.db.profile.BarTotemOrder[v]) do
			local totem = Yata:GetTotem(n)			
			local button = Yata.Button:Create(totem.Name,self.TotemButtons[totem.Element].Header, totem)
			button.Icon:SetTexture(totem.Texture)  	
			button:SetAttribute("*type*", "spell")
			button:SetAttribute("spell1", totem.Name)
			button:SetAttribute("spell2", GetSpellInfo(36936)) -- Totemic Call
			button:SetFrameRef("header",self.TotemButtons[totem.Element].Header)
			button:Execute([[header = self:GetFrameRef("header")]])	
			if LBF and self.LBFGroup then
				local group = self.LBFGroup
				button.LBFButtonData = {
					Button = button,
					Icon = button.Icon,
					HotKey = button.HotKey,
					Cooldown = button.Cooldown,
				}
				group:AddButton(button, button.LBFButtonData)
			end		
			button:SetAttribute("position", m-1)
			table.insert(self.TotemButtons.Buttons, button)
			table.insert(self.TotemButtons[totem.Element].Buttons,button)
		end
	end
	self:SetButtonSkin()
	self:SetBarConfiguration()
	self:SetAutoButtonSwap()
end

function Yata:BarUpdateHandler()
	if self.db.profile.TimerType == 1 then
		local totem = Yata.TotemData[GetSpellInfo(8071)] -- Stoneskin Totem
		local spellId = totem[#totem].SpellID
		local start, duration, enable = GetSpellCooldown(spellId, BOOKTYPE_SPELL)
		for k,v in ipairs(self.TotemButtons.Buttons) do
			CooldownFrame_SetTimer(v.Cooldown, start, duration, enable)
		end
	end
end

function Yata:StartMoving()
	Yata.TotemBarGrip.Texture:Show()
	for k,v in ipairs(elements) do
		self.TotemButtons[v].Header:Hide()
	end
	Yata:ShowTimerAnchors()
end
function Yata:StopMoving()
	Yata.TotemBarGrip.Texture:Hide()
	Yata:SaveBarConfig()
	for k,v in ipairs(elements) do
		self.TotemButtons[v].Header:Show()
	end
	Yata:HideTimerAnchors()	
end


function Yata:CreateBarGrip()
	if _G["YataBarGrip"] then return _G["YataBarGrip"] end
	local grip = CreateFrame("Frame", "YataBarGrip", UIParent, "SecureFrameTemplate")
	grip.Name = "YataBarGrip"
	grip.Parent = UIParent
	grip:SetHeight(36)
	grip:SetWidth(0)
	grip.Texture = grip:CreateTexture()
	grip.Texture:SetTexture(0, 0, 0.5, 0.5)
	grip.Texture:SetAllPoints(grip)	
	grip:RegisterForDrag( "LeftButton" )
	grip:EnableMouse(true)
	grip:SetMovable(true)
	grip:SetScript("OnDragStart", function()
		if(not self.db.profile.Locked) then
			this:StartMoving()
		end
	end)
	grip:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
	end)	
	return grip
end

function Yata:ColorYata()
--[[
	local overlayFrame = Yata:GetOverlayFrame()
	if (Yata.keyBoundMode) then
		overlayFrame:SetBackdropColor(LibKeyBound:GetColorKeyBoundMode())
	elseif (Yata.someOtherMode) then
	end
	--]]
end

function Yata:LIBKEYBOUND_ENABLED()
	for i,b in ipairs(elements) do
		self.TotemButtons[b].Header:SetAttribute("visiblebuttons",10)
		self.TotemButtons[b].Header:SetAttribute("show",true)
	end
	Yata.keyBoundMode = true
end

function Yata:LIBKEYBOUND_DISABLED()
	Yata.keyBoundMode = nil
	for i,b in ipairs(elements) do
		self.TotemButtons[b].Header:SetAttribute("visiblebuttons",self.db.profile.VisibleButtons)
		self.TotemButtons[b].Header:SetAttribute("show",false)
	end
end

function Yata:LIBKEYBOUND_MODE_COLOR_CHANGED()
end


function Yata:SaveBarConfig()
	if not self.db.profile.BarPosition then 
		self.db.profile.BarPosition = {}
	end
	local scale = self.TotemBarGrip:GetEffectiveScale()
	scale = scale / self.db.profile.BarScale
	local x = self.TotemBarGrip:GetLeft()/scale
	local y = self.TotemBarGrip:GetBottom()/scale 
	self.db.profile.BarPosition.x = x
	self.db.profile.BarPosition.y = y
	self.db.profile.BarTotemOrder.ElementsOrder = {}
	for k,v in ipairs(elements) do
		local header = self.TotemButtons[v].Header		
		local pos = math.floor((header:GetLeft() - self.TotemBarGrip:GetLeft())/36 + 0.5) + 1
		self.db.profile.BarTotemOrder.ElementsOrder[pos] = v
		self.db.profile.BarTotemOrder[v] = {}
		for i, b in ipairs(self.TotemButtons[v].Buttons) do
			self.db.profile.BarTotemOrder[v][b:GetAttribute("position")+1] = b.Totem.Name
		end
	end
	self:EditMacro(true,nil,nil)
end

function Yata:LoadBarConfig()
	if self.db.profile.Locked then
		self.TotemBarGrip.Texture:Hide()
	end	
	local x,y,scale
	if self.db.profile.BarPosition then
		scale = self.TotemBarGrip:GetEffectiveScale()	
		x = self.db.profile.BarPosition.x * scale
		y = self.db.profile.BarPosition.y * scale
	else
		self.TotemBarGrip:ClearAllPoints()
		self.TotemBarGrip:SetPoint( "CENTER", WorldFrame, "CENTER", 0,0 )
		scale = self.TotemBarGrip:GetEffectiveScale()
		x = self.TotemBarGrip:GetLeft()*scale
		y = self.TotemBarGrip:GetBottom()*scale
	end
	self.TotemBarGrip:ClearAllPoints()
	self.TotemBarGrip:SetPoint("BOTTOMLEFT", UIParent,"BOTTOMLEFT", x, y)
	self.TotemBarGrip:SetScale(self.db.profile.BarScale)	
end

function Yata:EditMacro(force, old,new)
	local num = GetNumMacros()
	local macroindex = GetMacroIndexByName("YataTotemStomp")
	if force or macroindex == 0 and num < 36 then
		local macro = "#showtooltip\n/castsequence reset=combat/"..self.db.profile.MacroResetKey.." "
		local totems = {}
		for k,v in ipairs(self.db.profile.BarTotemOrder.ElementsOrder) do
			for i,b in ipairs(self.TotemButtons[v].Buttons) do
				if b:GetAttribute("position") == 0 then
					table.insert(totems,b.Totem.Name)
					break
				end
			end
		end
		for k,v in ipairs(totems) do
			if totems[k] and totems[k+1] then
				macro = string.format("%s%s, ",macro,v)
			else
				macro = string.format("%s%s",macro,v)
			end
		end
		local numIcons = GetNumMacroIcons()
		local iconid = 0
		for i=1,numIcons do
			if GetMacroIconInfo(i) == "Interface\\Icons\\INV_Misc_QuestionMark" then
				iconid = i
				break
			end
		end	
		if force and macroindex > 0 then		
			EditMacro(macroindex, "YataTotemStomp", iconid, macro, 1)
		else
			CreateMacro("YataTotemStomp",iconid,macro,1,1)
		end
	elseif macroindex > 0 and old and new then
		local name, texture, macro, isLocal = GetMacroInfo(macroindex)
		macro = string.gsub(macro,old,new)
		EditMacro(macroindex, name, texture, macro, isLocal);
	end	
end